using System;
using System.Drawing;
using System.Drawing.Printing;
using System.Windows.Forms;
using System.Globalization;

namespace ReflectionIT.Common.Reporting
{

	/// <summary>
	/// NewPageHandler delegate
	/// </summary>
	public delegate void NewPageHandler(object sender, ReportItemCollection items, int page);

	/// <summary>
	/// Defines a reusable object for reporting data to a printer or previewing
	/// </summary>
	/// <remarks>
	/// Copyright  2002 Reflection IT
	/// 
	/// This software is provided 'as-is', without any express or implied warranty.
	/// In no event will the authors be held liable for any damages arising from the
	/// use of this software.
	/// 
	/// Permission is granted to anyone to use this software for any purpose,
	/// including commercial applications, subject to the following restrictions:
	/// 
	/// 1. The origin of this software must not be misrepresented; you must not claim
	/// that you wrote the original software. 
	/// 
	/// 2. No substantial portion of the source code of this library may be redistributed
	/// without the express written permission of the copyright holders, where
	/// "substantial" is defined as enough code to be recognizably from this library. 
	/// </remarks>
	public class Report : IDisposable
	{
		private string mTitle;
		private string mSubTitle;
		protected ReportItemCollection mItems;			
		protected Font mFont;
		protected PrintDocument mDocument;
		protected int mCurrentReportRow;
		protected int mPage;
		protected float mWidth  = 0.0f;

		/// <summary>
		/// Occurs when the a new page is printed.
		/// </summary>
		public event NewPageHandler NewPage;

		/// <summary>
		/// Constructor
		/// </summary>
		public Report()
		{
			// Intialize
			mItems = new ReportItemCollection();
			mDocument = new PrintDocument();
			mDocument.PrintPage += new PrintPageEventHandler(this.PrintPage);
			mDocument.BeginPrint += new PrintEventHandler(this.BeginPrint);

			// Defaults
			DefaultFont = new Font("Arial", 10);
			DefaultPageSettings.Margins.Top = 55;
			DefaultPageSettings.Margins.Bottom = 55;
			DefaultPageSettings.Margins.Left = 55;
			DefaultPageSettings.Margins.Right = 55;
		}

		/// <summary>
		/// Destuctor
		/// </summary>
		public void Dispose()
		{
			DefaultFont.Dispose();
			mDocument.Dispose();

			Items.Dispose();

			System.GC.SuppressFinalize(this);
		}

		/// <summary>
		/// Gets or sets the title of the report, is normally displayed in the header of the page
		/// </summary>
		public virtual string Title
		{
			get
			{
				return (this.mTitle);
			}
			set
			{
				this.mTitle = value;
				mDocument.DocumentName = mTitle;
			}
		}

		/// <summary>
		/// Gets or sets the subtitle of the report, is normally displayed in the header of the page
		/// </summary>
		public virtual string SubTitle
		{
			get
			{
				return (this.mSubTitle);
			}
			set
			{
				this.mSubTitle = value;
			}
		}

		/// <summary>
		/// Gets or sets the default font of the report. All items without a font use the default.
		/// </summary>
		public virtual Font DefaultFont
		{
			get
			{
				return (this.mFont);
			}
			set
			{
				this.mFont = value;
			}
		}

		/// <summary>
		/// Gets or sets page settings that are used as defaults for all pages to be reported.
		/// </summary>
		public PageSettings DefaultPageSettings
		{
			get { return mDocument.DefaultPageSettings; }
		}

		/// <summary>
		/// Gets or sets the printer that prints the report.
		/// </summary>
		public PrinterSettings PrinterSettings
		{
			get { return mDocument.PrinterSettings; }
		}

		/// <summary>
		/// Gets the collection of ReportItems
		/// </summary>
		public ReportItemCollection Items
		{
			get
			{
				return (this.mItems);
			}
		}

		/// <summary>
		/// Show the report in a default PrintPreview dialog
		/// </summary>
		public virtual void Preview()
		{

			this.Preview(new PrintPreviewDialog());
		}

		/// <summary>
		/// Show the report in a PrintPreview dialog
		/// </summary>
		/// <param name="dialog">PrintPreviewDialog in which the report is shown</param>
		public virtual void Preview(PrintPreviewDialog dialog)
		{
			dialog.Document = mDocument;
			dialog.ShowDialog();
		}

		/// <summary>
		/// Print the report without showing a PrintSetup dialog
		/// </summary>
		public virtual void Print()
		{
			mDocument.Print();
		}

		/// <summary>
		/// Print the report and allows users to select a printer and choose which portions of the document to print.
		/// </summary>
		/// <param name="dialog">PrintDialog</param>
		public virtual void Print(PrintDialog dialog)
		{
			mCurrentReportRow = 0;
			mPage = 0;
			
			dialog.Document = mDocument;

			if (dialog.ShowDialog() == DialogResult.OK)
				mDocument.Print();
		}

		protected void BeginPrint(object sender, PrintEventArgs e) {
			mCurrentReportRow = 0;
			mPage = 0;
		}

		/// <summary>
		/// PrintPage event handler
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="ev"></param>
		protected void PrintPage(object sender, PrintPageEventArgs ev) 
		{
			mPage++;
			float y = ev.MarginBounds.Top;

			OnNewPage(ev, ref y);

			while ((y < ev.MarginBounds.Bottom) && (mCurrentReportRow < Items.Count))
			{
				IReportItem item = Items[mCurrentReportRow];
				mCurrentReportRow ++;

				y += item.Print(ev, ev.MarginBounds.Left, y, DefaultFont);
			}

			ev.HasMorePages = (mCurrentReportRow < Items.Count - 1);
		}

		/// <summary>
		/// Raise NewPage event
		/// </summary>
		/// <param name="ev"></param>
		/// <param name="y"></param>
		protected void OnNewPage(PrintPageEventArgs ev, ref float y)
		{
			ReportItemCollection items = new ReportItemCollection();
			int x2 = 140;
			int x1 = ev.MarginBounds.Width - x2;
		
			// Create Box + Empty ReportGroup
			ReportGroup b = new ReportGroup(0, Width, 40);
			b.BorderPen = new Pen(Brushes.Black, 2);
			b.FixedTop = false;
			items.Add(b);
			items.Add(new ReportGroup(10));

			// Add a Empty ReportGroup to the Box
			ReportGroup r = new ReportGroup(4);
			b.Items.Add(r);

			// Add a 'Title + Page' ReportGroup to the Box
			r = new ReportGroup();
			b.Items.Add(r);
			ReportText i = new ReportText(Title, 5);
			i.Font = new Font(DefaultFont.FontFamily, DefaultFont.Size, FontStyle.Bold);
			r.Items.Add(i);
			r.Items.Add(new ReportText("Page :", x1));
			i = new ReportText(mPage.ToString(), x1, x2);
			i.Alignment = HorizontalAlignment.Right;
			r.Items.Add(i);

			// Add a 'SubTitle + Date' ReportGroup to the Box
			r = new ReportGroup();
			b.Items.Add(r);
			r.Items.Add(new ReportText(SubTitle, 20));
			r.Items.Add(new ReportText("Date :", x1));
			CultureInfo cu = CultureInfo.CurrentCulture;
			i = new ReportText(DateTime.Today.ToString(cu.DateTimeFormat.ShortDatePattern), x1, x2);
			i.Alignment = HorizontalAlignment.Right;
			r.Items.Add(i);

			// RaiseEvent
			if (NewPage != null)
			{
				// Add extra Items?
				NewPage(this, items, mPage);
			}

			// Print these items
			foreach (IReportItem item in items)
			{
				y += item.Print(ev, ev.MarginBounds.Left, y, DefaultFont);
			}
		}

		/// <summary>
		/// Returns the default width of the Report
		/// </summary>
		public float Width {
			get { 
				if (mWidth == 0.0 ) {
					mWidth = mDocument.DefaultPageSettings.Bounds.Width - mDocument.DefaultPageSettings.Margins.Right - mDocument.DefaultPageSettings.Margins.Left;
				}
				return  mWidth;
			}
		}

	}
}
