using System;
using System.Drawing;
using System.Drawing.Printing;
using System.Windows.Forms;

namespace ReflectionIT.Common.Reporting
{
	/// <summary>
	/// A ReportText is used to print a text on a specific X (Left) coordinate. 
	/// 
	/// You can use the 'Center' or 'Right' alignment when the Width property is set to a non-zero value.
	/// 
	/// A ReportText can have a BackColor and a BorderPen which only have meaning when 
	/// the Height property is set to a non-zero value
	/// </summary>
	public class ReportText : ReportItemBase, IReportItem
	{
		private String mText;
		private Brush mForeColor = Brushes.Black;
		private HorizontalAlignment mAlignment = HorizontalAlignment.Left;

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="text">Text</param>
		/// <param name="left">X-coordinate</param>
        public ReportText(string text, float left) : this(text, left, 0)
		{
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="text">Text</param>
		/// <param name="left">X-coordinate</param>
		/// <param name="width">Width</param>
		public ReportText(string text, float left, float width) : this(text, left, width, 0)
		{
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="text">Text</param>
		/// <param name="left">X-coordinate</param>
		/// <param name="width">Width</param>
		/// <param name="heigth">Heigth</param>
		public ReportText(string text, float left, float width, float heigth) : base(left, width, heigth)
		{
			this.Text = text;
		}

		/// <summary>
		/// Get or sets the text to print
		/// </summary>
		public String Text
		{
			get
			{
				return (this.mText);
			}
			set
			{
				this.mText = value;
			}
		}

		/// <summary>
		/// Gets or sets the foreground color of the text.
		/// </summary>
		public Brush ForeColor
		{
			get
			{
				return (this.mForeColor);
			}
			set
			{
				this.mForeColor = value;
			}
		}

		/// <summary>
		/// Get or sets the horizontal alignment of the Text. 
		/// Width must have a non-zero value when set to Center or Right.
		/// </summary>
		public HorizontalAlignment Alignment
		{
			get
			{
				return (this.mAlignment);
			}
			set
			{
				this.mAlignment = value;
			}
		}

		float IReportItem.Print(PrintPageEventArgs ev, float x, float y, Font font)
		{
			if (Font != null)
				font = Font;
			
			// Calculate size
			bool oldHeigth0 = (Heigth == 0);
			if (Heigth == 0) 
				Heigth = font.GetHeight(ev.Graphics) + Margin.Top + Margin.Bottom;
			float retH = Heigth;

			float w = Width;
			if (w == 0)
				Width = ev.Graphics.MeasureString(Text, font).Width + Margin.Left + Margin.Right;
			
			// Print BackColor + Border
			base.Print(ev, x, y, font);

			// Print Text
			RectangleF r = new RectangleF(Left + x + Margin.Left, y + Margin.Top, w - Margin.Right , Heigth - Margin.Bottom);
			if (r.Right <= ev.MarginBounds.Right)
			{
				StringFormat sf = new StringFormat();

				if (Alignment == HorizontalAlignment.Center)
					sf.Alignment = StringAlignment.Center;
				if (Alignment == HorizontalAlignment.Right)
					sf.Alignment = StringAlignment.Far;				

				ev.Graphics.DrawString(Text, font, ForeColor, r, sf);
			}
		
			// Restore Size
			if (oldHeigth0)
				Heigth = 0;

			Width = w;

			// Return used Height
			return retH;
		}
	}
}
